<?php

namespace App\OpenApi;

use ApiPlatform\OpenApi\Factory\OpenApiFactoryInterface;
use ApiPlatform\OpenApi\OpenApi;
use ApiPlatform\OpenApi\Model;
use Symfony\Component\HttpFoundation\Response;

final readonly class OpenApiFactory implements OpenApiFactoryInterface
{

    public function __construct(private OpenApiFactoryInterface $decorated)
    {
    }

    public function __invoke(array $context = []): OpenApi
    {
        $openApi = $this->decorated->__invoke($context);

        $this->addRefreshToken($openApi);
        $this->addSearchEndpoint($openApi);
        $this->addStatusEndpoint($openApi);  // Agregar el nuevo endpoint

        return $openApi;
    }

    private function addRefreshToken(OpenApi $openApi): void
    {
        $openApi
            ->getPaths()
            ->addPath( '/auth/refresh', (new Model\PathItem())->withPost(
                (new Model\Operation('postRefreshToken'))
                    ->withTags(['Login check'])
                    ->withResponses([
                        Response::HTTP_OK => [
                            'description' => 'Refresh token',
                            'content' => [
                                'application/json' => [
                                    'schema' => [
                                        'type' => 'object',
                                        'properties' => [
                                            'token' => [
                                                'type' => 'string',
                                                'readOnly' => true,
                                                'nullable' => false,
                                            ],
                                            'refreshToken' => [
                                                'type' => 'string',
                                                'readOnly' => true,
                                                'nullable' => false,
                                            ]
                                        ],
                                        'required' => ['token', 'refreshToken'],
                                    ],
                                ],
                            ],
                        ],
                    ])
                    ->withSummary('Create refresh token')
                    ->withRequestBody(
                        (new Model\RequestBody())
                            ->withDescription('The refresh token data')
                            ->withContent( new \ArrayObject([
                                'application/json' => new Model\MediaType(new \ArrayObject(new \ArrayObject([
                                    'type' => 'object',
                                    'properties' => [
                                        'refreshToken' => [
                                            'type' => 'string',
                                            'nullable' => false,
                                        ],
                                    ],
                                    'required' => ['refreshToken'],
                                ])))
                            ]))
                            ->withRequired(true)
                    )
            ));
    }
    private function addSearchEndpoint(OpenApi $openApi): void
    {
        $openApi
            ->getPaths()
            ->addPath('/search', (new Model\PathItem())->withGet(
                (new Model\Operation('getSearch'))
                    ->withTags(['Search'])
                    ->withResponses([
                        Response::HTTP_OK => [
                            'description' => 'Search results',
                            'content' => [
                                'application/json' => [
                                    'schema' => [
                                        'type' => 'object',
                                        'properties' => [
                                            'results' => [
                                                'type' => 'array',
                                                'items' => [
                                                    'type' => 'object',
                                                    'properties' => [
                                                        'id' => [
                                                            'type' => 'integer',
                                                            'example' => 1,
                                                        ],
                                                        'name' => [
                                                            'type' => 'string',
                                                            'example' => 'Item name',
                                                        ],
                                                    ],
                                                ],
                                            ],
                                        ],
                                    ],
                                ],
                            ],
                        ],
                    ])
                    ->withSummary('Search for items')
                    ->withParameters([
                        new Model\Parameter(
                            'query',
                            'query',
                            'Search query parameter',
                            true,
                            false,
                        ),
                    ])
            ));
    }

    private function addStatusEndpoint(OpenApi $openApi): void
    {
        $openApi
            ->getPaths()
            ->addPath('/og-boot/status', (new Model\PathItem())->withGet(
                (new Model\Operation('getStatus'))
                    ->withTags(['OgBoot'])
                    ->withResponses([
                        Response::HTTP_OK => [
                            'description' => 'Service status',
                            'content' => [
                                'application/json' => [
                                    'schema' => [
                                        'type' => 'object',
                                        'properties' => [
                                            'status' => [
                                                'type' => 'string',
                                                'example' => 'ok',
                                            ],
                                            'uptime' => [
                                                'type' => 'integer',
                                                'example' => 12345,
                                            ],
                                        ],
                                    ],
                                ],
                            ],
                        ],
                        Response::HTTP_SERVICE_UNAVAILABLE => [
                            'description' => 'Service unavailable',
                            'content' => [
                                'application/json' => [
                                    'schema' => [
                                        'type' => 'object',
                                        'properties' => [
                                            'error' => [
                                                'type' => 'string',
                                                'example' => 'Service is down',
                                            ],
                                        ],
                                    ],
                                ],
                            ],
                        ],
                    ])
                    ->withSummary('Get service status')
            ));
    }
}